# `cookie`

Блок реализует набор методов для работы с куки браузера (JS-свойство `document.cookie`).

Блок реализован в технологии `js` и подходит для использования в клиентских приложениях.

## Публичные методы блока

Доступен следующий набор методов:

### `get`

Метод служит для получения значения, хранящегося в куки браузера, для имени переданного аргументом (`{String}`).

Метод возвращает:

* декодированную с помощью `decodeURIComponent` строку со значением в случае успеха.
* `null` если переданное аргументом имя не было найдено.

```js
modules.require(['cookie'], function(cookie) {
    cookie.set('mycookie', 'foobar');
    console.log(cookie.get('mycookie')); // 'foobar'
    console.log(cookie.get('foo')); // null
});
```

### `set`

Метод служит для записи куки с заданным именем. Помимо имени и значения, методу можно передать хеш с дополнительными параметрами куки.

Метод принимет следующие аргументы:

* `name` `{String}` – имя куки.
* `val` `{String}` | `{null}` – строка со значением, ассоциируемым с именем. При установке в качестве значения `null` куки будет удалено.
* `options` `{Object}` – хеш с дополнительными параметрами. В хеше допустимы следующие свойства с ожидаемым типом данных значения:
    * `expires` `{Number}` – срок жизни куки в сутках. При отрицательном значении куки будет удалено. Альтернативно, можно передать в качестве значения сформированный объект даты (`new Date()`).
    * `path` `{String}` – путь от корня домена внутри которого будет доступно куки.
    * `domain` `{String}` – домен в явном виде. По умолчанию – текущий.
    * `secure` `{Boolean}` – логический флаг, указывающий на необходимость принудительного использования с куки шифрованного соединения SSL. По умолчанию SSL не требуется.

Метод устанавливает значение куки и возвращает объект `this`.

```js
modules.require(['cookie'], function(cookie) {
    cookie.set('mycookie', 'foobar', {
        expires : 1, // срок жизни одни сутки
        path : '/', // доступно для всех страниц домена
        secure : true // передавать куки только по SSL
    });
    console.log(cookie.get('mycookie')); // 'foobar'

    cookie.set('mycookie', null }); // удаляем куки
    console.log(cookie.get('mycookie')); // null
});
```

