# identify

Блок реализует функцию для создания уникальных идентификаторов, которая позволяет:

* создавать идентификаторы объектов.
* проверять наличие идентификатора у объектов.
* получать уникальную строку-идентификатор.

Блок `inherit` реализован в технологии `vanila.js` и подходит для использования как на клиенте, так и на сервере.

Функция принимает аргументами:

* `obj` `{Object}` – идентифицируемый объект.
* `onlyGet` `{Boolean}` – опциональный флаг для проверки наличия идентификатора у объекта. При значении `true` функция будет возвращать строку с идентификатором только если объект был заранее идентифицирован. По умолчанию `false`.

Функция возвращает строку с идентификатором, присвоенным для объекта. Кроме того, строка сохраняется в свойствах объекта, так что при последующих вызовах всегда будет возвращаться один и тот же идентификатор.

```js
modules.require(['identify'], function(Identify) {
    var a = {},
        b = {},
        identA = identify(a);

    console.log(identA === identify(a)); // true
    console.log(identA === identify(b)); // false
});
```

Для совместимости с IE функция использует в качестве идентификатора объекта значение свойства `uniqueID`, если оно доступно.

При вызове без аргументов, функция будет каждый раз возвращать строку с уникальным идентификатором.

```js
modules.require(['identify'], function(Identify) {
    var a = identify(),
        b = identify();

    console.log(a === b); // false
});
```
