# idle

Блок предназначен для генерации события в момент начала простоя окна браузера (когда пользователь работает с другим окном или не совершает действий). Подписавшись на события блока можно приостанавливать выполнение неких операций, например, отображение анимации, при отсутствии пользовательской активности.

Блок реализован в технологии `js` и подходит для использования в клиентских приложениях.

## События блока

Доступен следующий набор событий:

* `idle` – событие генерируется по истечению трех секунд после последней пользовательской активности. Под активностью подразумеваются события `mousemove`, `keydown` и `click`.
* `wakeup` – событие генерируется в момент возобновления пользовательской активности (события `mousemove`, `keydown` и `click`).

## Публичные методы блока

Доступен следующий набор методов:

* `start` – запуск генерации события и отслеживания простоя.
* `stop` – остановка генерации события и отслеживания простоя.
* `isIdle` – возвращает текущее состояние (`true` если окно простаивает).

Методы не имеют параметров.

Блок наследуется от «клaсса» `Emitter` блока `events`, что позволяет вызывать его методы.
Например, в блоке `input` производится подписка на события `idle` и `wakeup` с помощью унаследованного `idle.on`:

```js
modules.define('input', ['tick', 'idle'], function(provide, tick, idle, Input) {

var instances = [],
    boundToTick,
    bindToTick = function() {
        boundToTick = true;
        tick
            .on('tick', update) // подписываемся на событие tick 
            .start(); // запускаем генерацию события tick
        idle
            .on({
                idle : function() {
                    tick.un('tick', update); // по событию idle отписываемся от tick
                },
                wakeup : function() {
                    tick.on('tick', update); // по событию wakeup подписываемся обратно
                }
            })
            .start(); // запускаем генерацию события idle
    },
    update = function() { /* ... */ };

});
```
