# tick

Блок `tick` служит для генерации регулярного события `tick`. Подписавшись на событие блока, можно использовать его для поллинга и синхронизации выполняемых операций.

Блок реализован в технологии `vanilla.js` и подходит для использования как на клиенте, так и на сервере.

## Интервал между событиями

Частота, с которой блок генерирует событие `tick`, определяется значением приватной переменной `TICK_INTERVAL`. Значение переменной – целое число – интервал между событиями в миллисекундах. 

По умолчанию `TICK_INTERVAL = 50`. Не рекомендуется изменять значение переменной.

## Публичные методы блока

Блок предоставляет следующие публичные методы:

* `start` – запускает генерацию события `tick` и счетчик, если они еще не запущены. Событие `tick` генерируется через интервал, равный значению `TICK_INTERVAL` после вызова метода;
* `stop` – останавливает генерацию события.

Методы не принимают никаких аргументов.

```js
modules.define('tick-emmiter',
    ['i-bem__dom', 'tick'],
    function(provide, BEMDOM, tick) {
        
provide(BEMDOM.decl(this.name, {
    onSetMod : {
        'js' : {
            'inited' : function() { 
                this.setMod('ticking');
            },

            '' : function() {
                this.delMod('ticking');
            }
        },

        'ticking' : {
            'true' : function() {
                tick.start();
            },

            '' : function() {
                tick.stop();
            }
        }
    }
}));
});
```


Блок наследуется от «клсаса» `Emitter` блока `events`, что позволяет вызывать методы класса.
Например, в блоке `input` для поллинга производится подписка на событие `tick` помощью унаследованного `tick.on`:

```js
modules.define('input', ['tick', 'idle'], function(provide, tick, idle, Input) {

var instances = [],
    boundToTick,
    bindToTick = function() {
        boundToTick = true;
        tick
            .on('tick', update) // подписываемся на событие tick 
            .start(); // запускаем генерацию события tick
        idle
            .on({
                idle : function() {
                    tick.un('tick', update); // по событию idle отписываемся от tick
                },
                wakeup : function() {
                    tick.on('tick', update); // по событию wakeup подписываемся обратно
                }
            })
            .start();
    },
    update = function() { /* ... */ };

});
```


## Модификаторы блока

### start_auto

Блок с модификатором `start` в значении `auto` используется для автоматического запуска генерации события `tick`. Событие начинает генерироваться в момент инициализации блока.
